;;;; -*- lisp -*-

(in-package :it.bese.ucw-user)

;;;; the file uploading example and test

(defclass file-upload-example (widget-component template-component)
  ((file :accessor file-upload-example.file :initform ""))
  (:metaclass standard-component-class)
  (:default-initargs :template-name "ucw/examples/upload.tal")
  (:documentation "Form for uploading a file."))

(defaction upload ((form file-upload-example))
  (call 'file-upload-viewer :file-data (file-upload-example.file form)))

(defclass file-upload-viewer (widget-component)
  ((file-data :initarg :file-data))
  (:metaclass standard-component-class)
  (:documentation "View a file uplodaed by the file-upload-example component."))

(defmethod render ((viewer file-upload-viewer))
  (<:table
   (<:tr (<:th "Header") (<:th "Value"))
   (dolist* ((name . value) (mime-part-headers (slot-value viewer 'file-data)))
     (<:tr
      (<:td (<:as-html name)) (<:td (<:as-html value)))))
  (<:p "Body:")
  (<:pre (<:as-html (mime-part-body (slot-value viewer 'file-data))))
  (<ucw:a :action (ok viewer) "Ok."))

(defclass composite-field (composite-element)
  ((integer-field :accessor integer-field
                  :component (integer-element))
   (decimal-field :accessor decimal-field
                  :component (decimal-element
                              :default-value 5.47
                              :constraints '(:not-null))))
  (:metaclass standard-component-class))

(defclass example-form (form-element)
  ((integer-field :accessor integer-field
		  :component (integer-element))
   (decimal-field :accessor decimal-field
		  :component (decimal-element
                              :min-value 10
                              :max-value 20
                              :default-value 14.3
                              :constraints '(:not-null :number-range)))
   (text-field :accessor text-field
	       :component (completing-text-element
			   :completions '("alice" "almond" "april" "bob" "bill" "cynthia")
                           :label #'(lambda (e)
                                        (<:as-is "A string (with completion):")
                                        (<:br)
                                        (<:as-is "The default options are: ")
                                        (<:as-html (ucw::completions e))
                                        (<:as-is "Use the up and down arrows to chose a completion,")
                                        (<:as-is "enter to select it."))
			   :prefix "example-input"))
   (select-field :accessor select-field
                 :component (select-element :options '("alice" "almond" "april" "bob" "bill" "cynthia")))
   (time-field :accessor time-field
               :component (time-element))
   (time-field1 :accessor time-field1
                :component (time-element
                            :output-format "%d/%B/%Y"))
   (time-field2 :accessor time-field2
                :component (time-element
                            :output-format "%H:%M:%S"))
   (composite-field :accessor composite-field
                    :component (composite-field)))
  (:metaclass standard-component-class)
  (:default-initargs :label "Form Demo"))

(defaction change-edit-mode ((f example-form) mode)
  (setf (editablep f) mode))

(defmethod render-slots :after (view (form example-form) slots)
  (<:tr
   (<:td :colspan 2
     (if (editablep form)
         (progn
           (<ucw:input :type "submit" :action (submit form))
           (<ucw:input :type "submit" 
                       :action (change-edit-mode form nil)
                       :value "View Lisp Values"))
         (<ucw:input :type "submit"
                     :action (change-edit-mode form t)
                     :value "Continue Editing")))))

(defclass example-auxslots-form (auxslots-form-element)
  ((select-field :accessor select-field
                 :component (select-element
                             :options (mapcar #'(lambda (n)
                                                  (cons (ie-type-class n)
                                                        (symbol-name n)))
                                              '(text integer decimal))
                             :key #'car
                             :output-format #'(lambda (v e)
                                                (declare (ignore e)) (cdr v)))))
  (:metaclass standard-component-class))

(defaction add-auxslot ((f example-auxslots-form))
  (with-slots (select-field)
      f
    (setf (lisp-value select-field) (read-client-value select-field))
    (aif (lisp-value select-field)
         (setf (slot-value f (gensym (cdr (assoc it (ucw::options select-field)))))
               (make-instance it))))
  t)

(defmethod render ((form example-auxslots-form))
  (<ucw:form :action (submit form) :method "POST"
    (<:table
     (<:tr
        (<:td "Add new field of type")
        (<:td :colspan 2 (render (select-field form)))
        (<:td (<ucw:input :type "submit"
                          :action (add-auxslot form)
                          :value "Add")))
     (dolist* (slot-name (remove 'select-field (slot-elements form)))
       (let ((field (slot-value form slot-name)))
         (<:tr
          (<:td (<:as-html slot-name) ":")
          (<:td (render field))
          (<:td (inspect-anchor form (lisp-value field)))
          (<:td (<ucw:input :type "submit"
                            :action (slot-makunbound form slot-name)
                            :value "Delete")))))
     (<:tr
      (<:td :colspan 4 (<ucw:input :type "submit" :action (submit form)))))))

(defclass example-set-obj ()
  ((integer-field :accessor integer-field
                  :initarg :integer-field
                  :initform nil)
   (decimal-field :accessor decimal-field
		  :initarg :decimal-field
                  :initform nil)
   (text-field :accessor text-field
	       :initarg :text-field
               :initform nil)
   (select-field :accessor select-field
                 :initarg :select-field
                 :initform nil)
   (time-field :accessor time-field
               :initarg :time-field
               :initform nil)))

(defparameter +example-set-items+
  (loop
     with l = '("alice" "almond" "not in list" "april" "bob" "bill" "cynthia")
     with ll = (1- (length l))
     for s in l
     for i upfrom 10
     for d = (+ i 2.5)
     for tt = (nth (random ll) l)
     for tm = (+ (get-universal-time) (* 3600 24 5 i))
     collect (make-instance 'example-set-obj
                            :integer-field i
                            :decimal-field d
                            :text-field tt
                            :select-field s
                            :time-field tm)))

(defclass example-set-viewer (form-element)
  ((integer-field :accessor integer-field
		  :component (integer-element
                              :direct-value (indirect-value nil 'integer-field)))
   (decimal-field :accessor decimal-field
		  :component (decimal-element
                              :direct-value (indirect-value nil 'decimal-field)))
   (text-field :accessor text-field
	       :component (text-element
                           :direct-value (indirect-value nil 'text-field)))
   (select-field :accessor select-field
                 :component (select-element
                             :test #'equal
                             :options '("alice" "almond" "april" "bob" "bill" "cynthia")
                             :direct-value (indirect-value nil 'select-field)))
   (time-field :accessor time-field
                :component (time-element 
                            :direct-value (indirect-value nil 'time-field)
                            :output-format "%d/%B/%Y")))
  (:metaclass standard-component-class)
  (:default-initargs :label "CRUD Demo"))

(defaction submit ((form example-set-viewer))
  (when (validp form)
    (ok form)))

(defaction cancel ((form example-set-viewer))
  (answer nil))

(defmethod render-slots :after (view (form example-set-viewer) slots)
  (<:tr
    (<:td :colspan 2
      (<ucw:input :type "submit" :action (submit form) :value "Save")
      (<ucw:input :type "submit" :action (cancel form) :value "Cancel"))))

(defmethod (setf lisp-value) ((value example-set-obj) (f example-set-viewer))
  (mapc #'(lambda (slot-name)
            (let ((se (slot-value f slot-name)))
              (setf (slot-indirect-object se 'direct-value) value)
              (setf (client-value se) (lisp-value se))))
        (slot-elements f)))

(defclass example-set (range-set-element)
  ((item-editor :accessor item-editor
                :component (example-set-viewer
                            :label "Edit Item")
                :initarg :item-editor))
  (:metaclass standard-component-class)
  (:default-initargs
    :item-viewer (make-instance 'example-set-viewer)
    :window-size 3
    :data (copy-list +example-set-items+)))

(defaction edit-item ((set example-set) item)
  (setf (lisp-value (item-editor set)) item)
  (call-component set (item-editor set)))

(defaction new-item ((set example-set))
  (let ((obj (make-instance 'example-set-obj)))
    (setf (lisp-value (item-editor set)) obj)
    (when (call-component set (item-editor set))
      (add-item set obj)
      t)))

(defmethod shared-initialize :around ((set example-set) slot-names
                                      &key &allow-other-keys)
  (declare (ignore slot-names))
  (call-next-method)
  (ucw::scroll set :next))

(defmethod render-slot-labels :before (view (set example-set) slots)
  (declare (ignore view slots))
  (<:th ""))

(defmethod render-item :before (view (set example-set) item slots)
  (let ((index (rownum view)))
    (<:td (<ucw:a :action (edit-item set item)
                  (<:as-html "Edit"))
          (<ucw:a :action (delete-item set index)
                  (<:as-html "Delete"))
          )))

(defmethod render-items :after (view (set example-set) slot-names)
  (<:tr
   (<:td :colspan (1+ (length slot-names))
     (<ucw:a :action (new-item set) (<:as-html "New")))))

(defclass example-search (ucw::search-element)
  ()
  (:metaclass standard-component-class)
  (:default-initargs
   :label "Search"
   :attributes (make-instance 'example-set-viewer)
   :query-value '((select-field (:= "alice"))
                  (decimal-field (:> 2345))
                  (text-field (:not (:contains "bb"))))))

(defmethod present (view (form example-search))
  (<:tr (<:td (<:code (<:as-html (format nil "~a" (query-value form))))))
  (<:tr (<:td (<ucw:input :type "submit" :action (edit-query form) :value "Edit Query"))))


(defaction submit ((form example-search))
  (when (validp form)
    (setf (editablep form) nil)))

(defaction edit-query ((form example-search))
  (setf (editablep form) t))

(defmethod render-slots :after (view (form example-search) slots)
  (<:tr
    (<:td :colspan 5
       (<ucw:input :type "submit" :action (submit form) :value "View Query"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Copyright (c) 2003-2005 Edward Marco Baringer
;;; All rights reserved. 
;;; 
;;; Redistribution and use in source and binary forms, with or without
;;; modification, are permitted provided that the following conditions are
;;; met:
;;; 
;;;  - Redistributions of source code must retain the above copyright
;;;    notice, this list of conditions and the following disclaimer.
;;; 
;;;  - Redistributions in binary form must reproduce the above copyright
;;;    notice, this list of conditions and the following disclaimer in the
;;;    documentation and/or other materials provided with the distribution.
;;; 
;;;  - Neither the name of Edward Marco Baringer, nor BESE, nor the names
;;;    of its contributors may be used to endorse or promote products
;;;    derived from this software without specific prior written permission.
;;; 
;;; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
;;; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
;;; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
;;; A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
;;; OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
;;; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
;;; LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
;;; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
;;; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
;;; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
;;; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
